#include <cstdio>
#include "gic.h"

namespace Driver
{

Gic::Gic(Bus *bus, Address base_cpu, Address base_dist)
  : _bus(bus), _base_cpu(base_cpu), _base_dist(base_dist)
{
  wr_dist(DIST_CTRL, 0);

  unsigned num = num_irqs();
  printf("Number of IRQs available at this GIC: %d\n", num);

  // FIXME: get cpuid from somewhere
  unsigned cpu_id = 0;
  unsigned int intmask = 1 << cpu_id;
  intmask |= intmask << 8;
  intmask |= intmask << 16;

  for (unsigned i = 32; i < num; i += 16)
    wr_dist(DIST_CONFIG + i * 4 / 16, 0);
  for (unsigned i = 32; i < num; i += 4)
    wr_dist(DIST_TARGET + i, intmask);
  for (unsigned i = 0; i < num; i += 4)
    wr_dist(DIST_PRI + i, 0xa0a0a0a0);
  for (unsigned i = 0; i < num; i += 32)
    wr_dist(DIST_ENABLE_CLEAR + i * 4 / 32, 0xffffffff);

  wr_dist(DIST_CTRL, 1);

  wr_cpu(CPU_CTRL, CPU_CTRL_ENABLE);
  wr_cpu(CPU_PRIMASK, 0xf0);
}

unsigned
Gic::num_irqs()
{
  return ((rd_dist(DIST_CTR) & 0x1f) + 1) * 32;
}

void
Gic::enable(unsigned irq)
{
  wr_dist(DIST_ENABLE_SET + (irq / 32) * 4, 1 << (irq % 32));
}

void
Gic::disable(unsigned irq)
{
  wr_dist(DIST_ENABLE_CLEAR + (irq / 32) * 4, 1 << (irq % 32));
}

void
Gic::ack(unsigned irq)
{
  return wr_cpu(CPU_EOI, irq);
}

unsigned
Gic::pending()
{
  return rd_cpu(CPU_INTACK) & 0x3ff;
}

void
Gic::wr_dist(Address reg, Mword val)
{
  _bus->write32(_base_dist + reg, val);
}

void
Gic::wr_cpu(Address reg, Mword val)
{
  _bus->write32(_base_cpu + reg, val);
}

Mword
Gic::rd_dist(Address reg)
{
  return _bus->read32(_base_dist + reg);
}

Mword
Gic::rd_cpu(Address reg)
{
  return _bus->read32(_base_cpu + reg);
}

}
