#include "icu_i8259.h"

#include <cstdio>
#include <cassert>

namespace Driver
{
  I8259::I8259(Bus *bus, Address base_master, Unsigned8 vector_master,
                         Address base_slave, Unsigned8 vector_slave)
    : _bus(bus), _base_master(base_master), _base_slave(base_slave)
      {
	/* Initialize the master. */
	wr_master(ICW, PICM_ICW1);
	wr_master(OCW, vector_master);
	wr_master(OCW, PICM_ICW3);
	wr_master(OCW, PICM_ICW4);

	/* Initialize the slave. */
	wr_slave(ICW, PICS_ICW1);
	wr_slave(OCW, vector_slave);
	wr_slave(OCW, PICS_ICW3);
	wr_slave(OCW, PICS_ICW4);

	// set initial masks
	wr_master(OCW, 0xfb);	// unmask irq2
	wr_slave(OCW, 0xff);	// mask everything

	/* Ack any bogus intrs by setting the End Of Interrupt bit. */
	wr_master(ICW, NON_SPEC_EOI);
	wr_slave(ICW, NON_SPEC_EOI);
      }

  unsigned I8259::num_irqs()
    { return 16; }

  void I8259::disable(unsigned irq)
    {
      if (irq < 8)
	wr_master(OCW, rd_master(OCW) | (1 << irq));
      else
	wr_slave(OCW, rd_slave(OCW) | (1 << (irq-8)));
    }

  void I8259::enable(unsigned irq)
    {
      if (irq < 8)
	wr_master(OCW, rd_master(OCW) & ~(1 << irq));
      else
	{
	  wr_master(OCW, rd_master(OCW) & ~(1 << 2));
	  wr_slave(OCW, rd_slave(OCW) & ~(1 << (irq-8)));
	}
    }

  void I8259::ack(unsigned irq)
    {
      if (irq >= 8)
	wr_slave(ICW, NON_SPEC_EOI);

      wr_master(ICW, NON_SPEC_EOI);
    }

  unsigned I8259::pending()
    {
      return 0;
    }
    
  void I8259::wr_master(Address address, Unsigned8 val)
    {
      _bus->write8(_base_master + address, val);
    }

  Unsigned8 I8259::rd_master(Address address)
    {
      return _bus->read8(_base_master + address);
    }

  void I8259::wr_slave(Address address, Unsigned8 val)
    {
      _bus->write8(_base_slave + address, val);
    }

  Unsigned8 I8259::rd_slave(Address address)
    {
      return _bus->read8(_base_slave + address);
    }
}
