#pragma once

#include "bus.h"
#include "icu.h"

namespace Driver
{
  class Gic : public Icu
    {
      enum
	{
	  DIST_CTRL         = 0x000,
	  DIST_CTR          = 0x004,
	  DIST_IRQ_SEC      = 0x080,
	  DIST_ENABLE_SET   = 0x100,
	  DIST_ENABLE_CLEAR = 0x180,
	  DIST_PRI          = 0x400,
	  DIST_TARGET       = 0x800,
	  DIST_CONFIG       = 0xc00,
	  DIST_SOFTINT      = 0xf00,

	  CPU_CTRL          = 0x00,
	  CPU_PRIMASK       = 0x04,
	  CPU_BPR           = 0x08,
	  CPU_INTACK        = 0x0c,
	  CPU_EOI           = 0x10,
	  CPU_RUNINT        = 0x14,
	  CPU_PENDING       = 0x18,

	  CPU_CTRL_ENABLE          = 1,
	};

    public:
      Gic(Bus *bus, unsigned long _base_cpu, unsigned long _base_dist);
      unsigned num_irqs();
      void enable(unsigned irq);
      void disable(unsigned irq);
      void ack(unsigned irq);
      unsigned pending();

    protected:
      void wr_dist(Address reg, Mword val);
      void wr_cpu(Address reg, Mword val);
      Mword rd_dist(Address reg);
      Mword rd_cpu(Address reg);

    private:
      Bus *_bus;
      unsigned long _base_cpu;
      unsigned long _base_dist;
    };
}
