#pragma once

#include "bus.h"
#include "icu.h"

namespace Driver
{
  class I8259 : public Icu
  {
  public:
    enum
      {
	ICW = 0x00,
	OCW = 0x01,

	/* ICW1	*/
	ICW_TEMPLATE    = 0x10,
	LEVL_TRIGGER    = 0x08,
	EDGE_TRIGGER    = 0x00,
	ADDR_INTRVL4    = 0x04,
	ADDR_INTRVL8    = 0x00,
	SINGLE__MODE    = 0x02,
	CASCADE_MODE    = 0x00,
	ICW4__NEEDED    = 0x01,
	NO_ICW4_NEED    = 0x00,

	/* ICW3	*/
	SLAVE_ON_IR0    = 0x01,
	SLAVE_ON_IR1    = 0x02,
	SLAVE_ON_IR2    = 0x04,
	SLAVE_ON_IR3    = 0x08,
	SLAVE_ON_IR4    = 0x10,
	SLAVE_ON_IR5    = 0x20,
	SLAVE_ON_IR6    = 0x40,
	SLAVE_ON_IR7    = 0x80,

	/* ICW4	*/
	SNF_MODE_ENA    = 0x10,
	SNF_MODE_DIS    = 0x00,
	BUFFERD_MODE    = 0x08,
	NONBUFD_MODE    = 0x00,
	AUTO_EOI_MOD    = 0x02,
	NRML_EOI_MOD    = 0x00,
	I8086_EMM_MOD   = 0x01,
	SET_MCS_MODE    = 0x00,

	/* OCW1	*/
	PICM_MASK	= 0xFF,
	PICS_MASK	= 0xFF,

	/* OCW2	*/
	NON_SPEC_EOI    = 0x20,
	SPECIFIC_EOI    = 0x60,
	ROT_NON_SPEC    = 0xa0,
	SET_ROT_AEOI    = 0x80,
	RSET_ROTAEOI    = 0x00,
	ROT_SPEC_EOI    = 0xe0,
	SET_PRIORITY    = 0xc0,
	NO_OPERATION    = 0x40,

	/* OCW3	*/
	OCW_TEMPLATE    = 0x08,
	SPECIAL_MASK    = 0x40,
	MASK_MDE_SET    = 0x20,
	MASK_MDE_RST    = 0x00,
	POLL_COMMAND    = 0x04,
	NO_POLL_CMND    = 0x00,
	READ_NEXT_RD    = 0x02,
	READ_IR_ONRD    = 0x00,
	READ_IS_ONRD    = 0x01,

	/* Standard PIC initialization values for PCs */
	PICM_ICW1	= (ICW_TEMPLATE | EDGE_TRIGGER | ADDR_INTRVL8 | CASCADE_MODE | ICW4__NEEDED),
	PICM_ICW3	= SLAVE_ON_IR2,
	PICM_ICW4	= (SNF_MODE_DIS | NONBUFD_MODE | NRML_EOI_MOD | I8086_EMM_MOD),

	PICS_ICW1	= (ICW_TEMPLATE | EDGE_TRIGGER | ADDR_INTRVL8 | CASCADE_MODE | ICW4__NEEDED),
	PICS_ICW3	= 0x2,
	PICS_ICW4	= (SNF_MODE_DIS | NONBUFD_MODE | NRML_EOI_MOD | I8086_EMM_MOD),
      };

    I8259(Bus *bus, Address base_master, Unsigned8 vector_master,
	            Address base_slave, Unsigned8 vector_slave);

    unsigned num_irqs();
    void disable(unsigned irq);
    void enable(unsigned irq);
    void ack(unsigned irq);
    unsigned pending();

  protected:
    void wr_master(Address address, Unsigned8 val);
    Unsigned8 rd_master(Address address);
    void wr_slave(Address address, Unsigned8 val);
    Unsigned8 rd_slave(Address address);

  private:
    Bus *_bus;
    Address _base_master;
    Address _base_slave;
  };
}
