#pragma once

#include "uart.h"
#include "bus.h"

namespace Driver
{
  class Uart_16550 : public Uart
  {
  public:

    enum
      {
	TRB      = 0, // Transmit/Receive Buffer  (read/write)
	BRD_LOW  = 0, // Baud Rate Divisor LSB if bit 7 of LCR is set  (read/write)
	IER      = 1, // Interrupt Enable Register  (read/write)
	BRD_HIGH = 1, // Baud Rate Divisor MSB if bit 7 of LCR is set  (read/write)
	IIR      = 2, // Interrupt Identification Register  (read only)
	FCR      = 2, // 16550 FIFO Control Register  (write only)
	LCR      = 3, // Line Control Register  (read/write)
	MCR      = 4, // Modem Control Register  (read/write)
	LSR      = 5, // Line Status Register  (read only)
	MSR      = 6, // Modem Status Register  (read only)
	SPR      = 7, // Scratch Pad Register  (read/write)

	PAR_NONE = 0x00,
	PAR_EVEN = 0x18,
	PAR_ODD  = 0x08,
	DAT_5    = 0x00,
	DAT_6    = 0x01,
	DAT_7    = 0x02,
	DAT_8    = 0x03,
	STOP_1   = 0x00,
	STOP_2   = 0x04,

	MODE_8N1 = PAR_NONE | DAT_8 | STOP_1,
	MODE_7E1 = PAR_EVEN | DAT_7 | STOP_1,
      };

    Uart_16550(Bus *bus, Address base);

    virtual bool startup();
    virtual void shutdown() {};
   
    virtual int write(char const *s, unsigned long count);
 
  protected:
    /**
     * Read register of uart
     * \param reg	offset of register
     * \return		read value
     */
    Unsigned8 rd(Address reg);
    /**
     * Write register of uart
     * \param reg	offset of register
     * \param val	value to write
     */
    void wr(Address reg, Unsigned8 val);

  private:
    Bus *_bus;
    Address _base;
  };
}
