#include <cstdio>
#include <cstring>
#include "cpu.h"
#include "space.h"
#include "thread.h"

void
Cpu::init()
{
  extern char ivt_start, ivt_end;
  printf("ivt_start:%p ivt_end:%p\n", &ivt_start, &ivt_end);
  memcpy((char *)0, &ivt_start, &ivt_end - &ivt_start);
  
  unsigned long r;
  asm volatile("mrc p15, 0, %0, c1, c0, 0" : "=r" (r) : : "memory");
  r |= 2; // alignment check on
  asm volatile("mcr p15, 0, %0, c1, c0, 0" : : "r" (r) : "memory");
}

void
Cpu::switch_to(const Space *space)
{
  Mword domains = 0x55555555;
  Mword control = 1;
 
  asm volatile (
      "mcr p15, 0, %[asid], c13, c0, 1  \n" // ASID
      "mcr p15, 0, %[null], c8, c7, 0   \n" // TLB flush
      "mcr p15, 0, %[doms], c3, c0      \n" // domains
      "mcr p15, 0, %[pdir], c2, c0      \n" // pdbr
      "mcr p15, 0, %[control], c1, c0   \n" // control
      "mrc p15, 0, r0, c2, c0, 0        \n" // arbitrary read of cp15
      "mov r0, r0                       \n" // wait for result
      "sub pc, pc, #4                   \n"
      : :
      [asid]	"r"(space->id()),
      [pdir]    "r"(space->pdir().phys()),
      [doms]    "r"(domains),
      [control] "r"(control),
      [null]    "r"(0)
      : "r0"
      );
}

void
Cpu::current_thread(Thread *thread)
{
  extern char crt0_stack_high;
  _current = thread;

  asm volatile(
      "mov	sp, %0		\n" // reset stack pointer
      : 
      : "r" (&crt0_stack_high)
      : "memory"
      );

  thread->exec();
}

Thread *
Cpu::current_thread()
{
  return _current;
}

Page_fault
Cpu::page_fault(Mword data)
{
  register Mword ifar asm("r0") = 0;
  register Mword dfar asm("r1") = 0;
  register Mword mode asm("r2") = 0;

  asm volatile (
      "mrc p15, 0, %1, c6, c0, 2	\n"	// read IFAR
      "mrc p15, 0, %1, c6, c0, 0	\n"	// read DFAR
      "cps #0x17			\n"
      "mrs %2, spsr			\n"
      "cps #0x13			\n"
      : "=r" (ifar), "=r" (dfar), "=r"(mode)
      );
  mode &= 0x1f;

  Page_fault page_fault(mode == 0x10, data == 0 ? ifar : dfar);
  return page_fault;
}

#if 0
Page_fault
Cpu::data_page_fault()
{
  register Mword dfsr asm("r0") = 0;
  register Mword dfar asm("r1") = 0;
  register Mword mode asm("r2") = 0;

  asm volatile (
      "mrc     p15, 0, %0, c5, c0, 0	\n"	// read DFSR 
      "cps #0x17			\n"
      "mrs %2, spsr			\n"
      "cps #0x13			\n"
      : "=r" (dfsr), "=r" (dfar), "=r"(mode)
      );

  Page_fault page_fault(mode == 0x10, dfar);
  return page_fault;
}
#endif
