#include <stdio.h>
#include "exit.h"
#include "platform.h"
#include "thread.h"

extern "C"
void
do_entry_undef()
{
  Thread *thread = Platform::cpu()->current_thread();
  printf("---> entry_undef:%d\n", thread->id());
  thread->dump_user_regs();

  while (1);
}

extern "C"
void
do_entry_swi()
{
  Thread *thread = Platform::cpu()->current_thread();
  printf("---> entry_swi:%d\n", thread->id());
  thread->dump_user_regs();
  
  thread->handle_syscall();
  
  ret_to_user();
}

extern "C"
void
do_entry_inst_abort()
{
  Thread *thread = Platform::cpu()->current_thread();
  printf("---> entry_inst_abort:%d\n", thread->id());
  thread->dump_user_regs();

  Page_fault pf = Platform::cpu()->page_fault(0);

  if (pf.is_user())
    {
      printf("user pagefault %lx\n", pf.addr());
      thread->handle_user_fault(pf.addr());
    }
  else
    {
      printf("kernel pagefault\n");
      thread->handle_kernel_fault(pf.addr()); 
    }

  ret_to_user();
}

extern "C"
void
do_entry_data_abort()
{
  Thread *thread = Platform::cpu()->current_thread();
  printf("---> entry_data_abort:%d\n", thread->id());
  thread->dump_user_regs();
  
  Page_fault pf = Platform::cpu()->page_fault(1);

  if (pf.is_user())
    {
      printf("user pagefault %lx\n", pf.addr());
      thread->handle_user_fault(pf.addr());
    }
  else
    {
      printf("kernel pagefault\n");
      thread->handle_kernel_fault(pf.addr()); 
    }

  ret_to_user();
}

enum
{
  Irq_timer0 = 36,
};

extern "C"
void
do_entry_irq()
{
  Thread *thread = Platform::cpu()->current_thread();
  printf("---> entry_irq:%d\n", thread->id());
  thread->dump_user_regs();

  unsigned irq = Platform::icu()->pending();
  if (irq == Irq_timer0)
    {
      Platform::timer()->ack();
      Platform::icu()->ack(irq);
      thread->handle_timer_irq();
    }
  
  ret_to_user();
}
