#define THREAD_USER_STATE_OFFSET 8

// copied from user_regs fields
#define USR_SP		13 * 4
#define USR_LR		14 * 4
#define USR_PC		15 * 4
#define USR_CPSR	16 * 4

.macro GET_USER_STATE r
	ldr	\r, [sp]
	add	\r, \r, #THREAD_USER_STATE_OFFSET
.endm

.macro SAVE_USER_STATE
	stmia	lr, {r0 - r12}
	add	lr, lr, #USR_SP
	stmia	lr, {sp, lr}^
	sub	lr, lr, #USR_SP
	ldr	r0, [sp, #-4]
	str	r0, [lr, #USR_PC]
	ldr	r0, [sp, #-8]
	str	r0, [lr, #USR_CPSR]
.endm

.macro RESTORE_USER_STATE
	ldr	r0, [lr, #USR_CPSR]
	str	r0, [sp, #-8]
	ldr	r0, [lr, #USR_PC]
	str	r0, [sp, #-4]
	add	lr, lr, #USR_SP
	ldmia	lr, {sp, lr}^
	sub	lr, lr, #USR_SP
	ldmia	lr, {r0 - r12}
.endm

.section .vect,"a"
.globl exception_vector_table
exception_vector_table:
	nop				
	b	entry_undef
	b	entry_swi
	b	entry_iabort
	b	entry_dabort
	nop
	b	entry_irq
	b	entry_fiq

entry_undef:
	cps	#0x13
	str	lr, [sp, #-4]
	mrs	lr, spsr
	str	lr, [sp, #-8]
	GET_USER_STATE lr
	SAVE_USER_STATE
	adr	r1, undef_handler
	ldr	pc, [r1]

entry_swi:
	cps	#0x13
	str	lr, [sp, #-4]
	mrs	lr, spsr
	str	lr, [sp, #-8]
	GET_USER_STATE lr
	SAVE_USER_STATE
	adr	r1, swi_handler
	ldr	pc, [r1]

entry_iabort:
	cps	#0x13
	str	lr, [sp, #-4]
	mrs	lr, spsr
	str	lr, [sp, #-8]
	GET_USER_STATE lr
	SAVE_USER_STATE
	adr	r1, inst_abort_handler
	ldr	pc, [r1]
	
entry_dabort:
	cps	#0x13
	str	lr, [sp, #-4]
	mrs	lr, spsr
	str	lr, [sp, #-8]
	GET_USER_STATE lr
	SAVE_USER_STATE
	adr	r1, data_abort_handler
	ldr	pc, [r1]

entry_irq:
entry_fiq:
	sub	lr, lr, #4	// adjust saved ip
	str	lr, [sp, #-4]
	mrs	lr, spsr
	str	lr, [sp, #-8]
	cps	#0x13
	GET_USER_STATE lr
	SAVE_USER_STATE
	adr	r1, irq_handler
	ldr	pc, [r1]

undef_handler:
	.word	do_entry_undef
swi_handler:
	.word	do_entry_swi
inst_abort_handler:
	.word	do_entry_inst_abort
data_abort_handler:
	.word	do_entry_data_abort
irq_handler:
	.word	do_entry_irq

.section .text
.globl __ret_to_user
__ret_to_user:
	GET_USER_STATE lr
	RESTORE_USER_STATE
	ldr	lr, [sp, #-8]
	msr	spsr, lr
	ldr	lr, [sp, #-4]
	movs	pc, lr
