#pragma once

#include "types.h"
#include "ptab_attr.h"

class Pdir : public Root_page_table
{
  enum
    {
      Type_invalid	= 0x0,
      Type_coarse	= 0x1,
      Type_section	= 0x2,
    };

  struct Coarse
    {
      Mword type        : 2; // must be 0x01
      Mword sbz1        : 3;
      Mword domain      : 4;
      Mword p           : 1;
      Mword base        : 22;

      Address get_base()
	{ return (base << 10); }
      void set_base(Address pa)
	{ base = (pa >> 10); }
    };

  struct Section
    {
      enum
	{
	  Ap_kern	= 0x2,
	  Ap_user	= 0x3,
	};

      Mword type        : 2; // must be 0x02
      Mword buffered    : 1;
      Mword cached	: 1;
      Mword sbz1        : 1;
      Mword domain      : 4;
      Mword sbz2        : 1;
      Mword ap          : 2;
      Mword sbz3	: 5;
      Mword local	: 1;
      Mword sbz4        : 2;
      Mword base        : 12;

      void set_base(Address pa)
	{ base = (pa >> 20); }
    };

public:
  
  enum
    {
      Shift	= 20,
      Size	= 0x4000,
    };
  
  Pdir(Mword *raw);

  Address phys() const;

  Mword get_entry(unsigned index) const
    { return _raw[index]; }
  void set_entry(unsigned index, Mword entry)
    { _raw[index] = entry; }

  unsigned index(Address va)
    { return va >> Shift; }

  virtual void map_1mb(Address va, Address pa, Attr attr);
  virtual void map_4kb(Address va, Address pa, Attr attr);

  virtual Map &get_map(unsigned index);

private:
  Mword *_raw;
};

class Ptab
{
  enum
    {
      Type_invalid	= 0x0,
      Type_small	= 0x2,
    };

  struct Small
    {
      enum
	{
	  Ap_kern	= 0x2,
	  Ap_user	= 0x3,
	};
      
      Mword type        : 2; // must be 0x02
      Mword buffered    : 1;
      Mword cached	: 1;
      Mword ap0         : 2;
      Mword ap1         : 2;
      Mword ap2         : 2;
      Mword ap3         : 2;
      Mword base        : 20;

      void set_base(Address pa)
	{ base = (pa >> Shift); }
    };

public:
  
  enum
    {
      Shift	= 12,
      Mask	= 0xff,
      Size	= 0x1000,
    };

  Ptab(Mword *_raw);

  void map(Address va, Address pa, Attr attr);

private:
  unsigned index(Address va)
    { return (va >> Shift) & Mask; }
  
  Mword *_raw;
};
