#pragma once

#include "types.h"

class Space;
class Thread;

class Page_fault
{
public:
  Page_fault(bool is_user, Address addr)
    : _is_user(is_user), _addr(addr)
    {}
  bool is_user() const
    { return _is_user; }
  Address addr() const
    { return _addr; }

private:
  bool _is_user;
  Address _addr;
};

///< processor context
class Cpu
{
public:
  /**
   * Initialize hardware data structures
   */
  void init();
  /**
   * Switch to another space
   *
   * \param space	pointer of destination space
   */
  void switch_to(const Space *space);
  
  /**
   * get current thread
   *
   * \return		current thread
   */
  Thread *current_thread();

  /**
   * make another thread the current thread
   *
   * \param thread	pointer to new current thread
   */
  void current_thread(Thread *thread);

  /**
   * Generate a page fault object
   *
   * \param hint	architecture specific information
   */
  Page_fault page_fault(Mword hint);
};
