#include "types.h"

/// saved state of user mode
class User_state
{
public:
  /**
   * Returns the error code
   */
  Mword err();
  /**
   * Constructor
   * \param pc		initial user program counter
   */
  User_state(Mword pc);
  /**
   * print user state
   */
  void dump();
} __attribute__((packed));


//----------------------------------------------------------------------------

/// special interpretation of user state for system calls
class Sys_state : public User_state
{
public:
  /**
   * Returns system call number
   */
  unsigned nr();

  /**
   * Set return value
   * \param ret		new return value
   */
  void ret(int ret);
};

//----------------------------------------------------------------------------

/// user state for log system call
class Log_sys_state : public Sys_state
{
public:
  /**
   * Returns number bytes to output
   */
  unsigned num();

  /**
   * Returns true if the buffer should be flushed
   */
  bool flush();

  /**
   * Returns an output  character
   * \param i	i-th output character
   */
  char data(unsigned i);
};

//----------------------------------------------------------------------------

/// user state for IPC system call
class Ipc_sys_state : public Sys_state
{
public:
  /**
   * Returns destination thread id for IPC system calls
   */
  unsigned dest();
  
  /**
   * Copy parameter to destination system call state
   * \param dest	destination syscall state
   */
  void copy_to(Ipc_sys_state *dest);
};

