#include "platform.h"
#include "icu_i8259.h"
#include "timer_sp804.h"
#include "timer_i8254.h"
#include "uart_16550.h"
#include "bus.h"

#include "mem_layout.h"

enum
{
  Irq_pit = 0,
};

static
Driver::Bus *bus()
{
  static Driver::Io_bus _bus;
  return &_bus;
}

Cpu *
Platform::cpu()
{
  static Cpu _cpu;
  return &_cpu;
}

Driver::Uart *
Platform::uart()
{
  static Driver::Uart_16550 _uart(bus(), 0x3f8);
  return &_uart;
}

Driver::Icu *
Platform::icu()
{
  static Driver::I8259 _icu( bus(), 0x20, 0x28, 0xa0, 0x30);
  return &_icu;
}

Driver::Timer *
Platform::timer()
{
  static Driver::Timer_i8254 _i8254(bus(), 0x40);
  return &_i8254;
}

void
Platform::init()
{
  uart()->startup();
  cpu()->init();
  // set timer frequency
  timer()->init(32);
  icu()->enable(Irq_pit);
}

