#pragma once

#include "types.h"

/// attributes for a page table mapping
enum Attr
{
  Kern		= 0x1, ///< kernel-only accessible
  Device	= 0x2, ///< device memory
  User		= 0x3, ///< user memory
};

/// attributes for the mapping size
enum
{
  Large = 0,	///< Large entry, aka superpage
  Small = 1,	///< Small entry, aka page
};

struct Map;

/// interface for a top-level page table
class Root_page_table
{
public:
  /**
   * Get a reference to a mapping function
   *
   * \param index	size that should be mapped(i.e large or small)
   * \return		reference to mapping function
   */
  virtual Map &get_map(unsigned index) = 0;
};

/// Mapping function
struct Map
{
  unsigned size; ///< size that is mapped
  /**
   * Member function pointer to mapping function
   * \param va		virtual address
   * \param pa		physical address
   * \param attr	attributes of the mapping
   */
  void (Root_page_table::*map)(Address va, Address pa, Attr attr);
};
