#include "platform.h"
#include "gic.h"
#include "uart_pl011.h"
#include "timer_sp804.h"

#include "mem_layout.h"

enum
{
  Irq_timer0 = 36,
  Irq_timer1 = 37,
};

static
Driver::Bus *bus()
{
  static Driver::Mem_bus _bus;
  return &_bus;
}

Cpu *
Platform::cpu()
{
  static Cpu _cpu;
  return &_cpu;
}

Driver::Uart *
Platform::uart()
{
  static Driver::Pl011 _pl011(bus(), Mem_layout::Phys_uart0_base);
  return &_pl011;
}

Driver::Icu *
Platform::icu()
{
  static Driver::Gic _gic(bus(), Mem_layout::Phys_gic_cpu_base, Mem_layout::Phys_gic_dist_base);
  return &_gic;
}

Driver::Timer *
Platform::timer()
{
  static Driver::Sp804 _sp804(bus(), Mem_layout::Phys_timer0_base);
  return &_sp804;
}

void
Platform::init()
{
  cpu()->init();
  uart()->startup();
  icu()->enable(Irq_timer0);
  timer()->init(104999);
}

