#pragma once

#include "types.h"
#include "ptab.h"
#include "ptab_attr.h"

class Space
{
public:
  /**
   * Return the initial root space
   *
   * \returns		pointer to root space
   */
  static Space *root();

  /**
   * Constructor for root space.
   */
  Space();

  /**
   * Constructor that initializes space
   * \param space	pointer to root space
   */
  Space(Space *space);

  /**
   * Operator to allocate memory for a space
   */
  void *operator new(size_t size) throw();

  /**
   * Initialize root space mappings.
   */
  static void init_root();

  /**
   * Return global id of this space
   * \returns 		global id
   */
  unsigned id() const
    { return 0; }
  /**
   * Returns page directory of this space.
   *
   * \returns		reference to page directory		
   */
  const Pdir &pdir() const;

  /**
   * Map a virtual address range to a physical address range.
   *
   * \param va_start	start of the virtual address range
   * \param va_end	end of the virtual address range
   * \param pa		start of the physical address range
   * \param attr	attributes of the mapping
   * \param map_large	Map with large or small pages
   */
  void map(Address va_start, Address va_end, Address pa, Attr attr,  bool map_large);

  /**
   * Copy page table entries from source space to this space
   * 
   * \param space	pointer of source space to copy from
   * \param idx_start	page table index to start with copy
   * \param idx_end	page table index to end with copy
   */
  void copy_from(const Space *space, unsigned idx_start, unsigned idx_end);

private:
  // XXX implement me!
};
