#include "types.h"
#include "space.h"
#include "user_state.h"

class Thread;

//----------------------------------------------------------------------------

/**
 * Functions that return and change the current running thread
 */

/* return running thread */
static inline
Thread *
current()
{
  // XXX implement me!
  return 0;
}

/* make another thread the current thread */
static inline
void
current(const Thread *thread)
{
}

//----------------------------------------------------------------------------

/**
 * Present list of all threads in the system
 */
class Thread_list
{
public:
  /**
   * add a new thread
   * \param thread	pointer of new thread
   */
  void add(Thread *thread);
  /**
   * find thread in list
   * \param id		unique identifier of thread
   * \returns		pointer of thread or null
   */
  Thread *find(unsigned id);

private:
  // XXX implement me!
};

//----------------------------------------------------------------------------

/**
 * Represents a thread in the system.
 */
class Thread
{
public:
  /**
   * Visible states a thread can be in.
   */
  enum
    {
      State_idle = 2,		///< idling in the kernel
      State_ret_to_user,	///< thread should exit the kernel and enter user mode
      State_ipc_send,		///< is blocked in a send operation
      State_ipc_recv,		///< is blocked in a receive operation
    };

  /**
   * Constructor.
   *
   * \param state	initial state of this thread
   * \param space	pointer to the address space the thread is running in
   * \param pc		initial user program counter
   */
  Thread(Mword state, Space *space, Mword pc = 0);

  /**
   * Operator to allocate memory for a thread
   */
  void *operator new(size_t size) throw();
  
  /**
   * Get unique id
   * \returns		unique id
   */
  unsigned id() const;
  /**
   * Get current state
   * \returns		current state
   */
  Mword state() const;
  /**
   * Get space
   * \returns		pointer to space
   */
  Space *space() const;
  /**
   * Set a new state
   * \param new_state	value of new state
   */
  void state(Mword new_state);

  /**
   * Get saved user state
   * \returns		pointer to saved user state
   */
  User_state *user();
  /**
   * Get saved syscall state
   * \returns		pointer to saved syscall state
   */
  Sys_state *sys();

  /**
   * Get next thread in ready list
   * \returns		pointer to next thread
   */
  Thread *next();
  /**
   * Invoke the scheduler
   */
  void schedule();
 
  /**
   * Enqueue this thread into ready list
   */
  void enqueue();
  /**
   * Remove this thread from ready list
   */
  void dequeue();

  /**
   * Enqueue a thread into sender list
   * \param thread		pointer of thread to enqueue
   */
  void sender_enqueue(Thread *thread);
  /**
   * Dequeue a thread from sender list
   * \param thread		pointer of thread to dequeue
   */
  void sender_dequeue(Thread *thread);

  /**
   * Switch to another thread
   * \param next	thread to switch to
   */
  void switch_to(Thread *next);

  /**
   * Execute this thread.
   * Assertion: This thread should be current.
   */
  void exec();

  /**
   * Execute log system call
   */
  void log();

  /**
   * Execute IPC send system call.
   */
  void ipc_send();
  /**
   * Execute IPC receive system call.
   */
  void ipc_recv();
  /**
   * Dispatch a system call for this thread.
   */
  void handle_syscall();
  
  /**
   * Handle a user page fault for this thread.
   *
   * \param addr	address where the page fault occurred.
   */
  void handle_user_fault(Address addr);
  /**
   * Handle a kernel page fault for this thread.
   *
   * \param addr	address where the page fault occurred.
   */
  void handle_kernel_fault(Address addr);

  /**
   * Handle a timer interrupt.
   *
   */
  void handle_timer_irq();

  /**
   * Show the user state of this thread.
   */
  void dump_user_regs();

protected:
  /**
   * Copy registers of user state from this thread to destination thread
   * \param dest	pointer to destination thread
   */
  void _copy_to(Thread *dest);

private:
  // XXX implement me!
};
