#include <stdio.h>
#include <string.h>
#include "thread.h"
#include "user.h"

Region_manager::Region_manager()
{
  _num_regions = 0;
}

void
Region_manager::add(unsigned id, Address virt, Address phys, unsigned size)
{
  printf("[rm]: add region: [%08lx:%08lx] => [%08lx:%08lx]\n",
      virt, virt + size, phys, phys + size);
  
  if (_num_regions < Max_regions)
    {
      Region &region = _region_map[_num_regions];
      region.id = id;
      region.phys = phys;
      region.virt = virt;
      region.size = size;
      _num_regions++;
      return;
    }

  printf("[rm]: Error: out of regions\n");
}

Address
Region_manager::translate(unsigned id, Address virt)
{
  printf("[rm]: translate:%d %lx %d\n", id, virt, _num_regions);
  for (unsigned i = 0; i < _num_regions; ++i)
    {
      Region &region = _region_map[i];
      if ((id == region.id) &&
	  (virt >= region.virt) && (virt < (region.virt + region.size)))
	return region.phys + (virt - region.virt);
    }
  return 0;
}

void
User_binary::init(unsigned id, unsigned start, unsigned end)
{
  _id = id;
  _start = start;
  _end = end;
}

void *
User_binary::start()
{ return (void *)_start; }

void
User_binary::load(unsigned p_offset, unsigned p_filesz, unsigned p_paddr,
                  unsigned p_vaddr, unsigned p_memsz, int type)
{
  // create new region
  Region_manager::rm()->add(_id, p_vaddr, _start + p_offset, p_filesz);
}

struct mod_info
{
  Unsigned32 start;
  Unsigned32 size;
  Unsigned32 size_uncompressed;
  Unsigned32 name;
  Unsigned32 md5sum_compr;
  Unsigned32 md5sum_uncompr;
};

void Loader::exec()
{
  extern mod_info _module_info_start[], _module_info_end[];
  
  printf("[ldr] module info: [%08lx:%08lx]\n", (Mword)&_module_info_start, (Mword)&_module_info_end);
  
  unsigned mods_count  = _module_info_end - _module_info_start;
  static User_binary _binary;

  
  for (unsigned i = 0; (i < mods_count); ++i)
    {
      Unsigned32 mod_start = _module_info_start[i].start;
      Unsigned32 mod_end   = mod_start + _module_info_start[i].size_uncompressed;
      printf("[ldr]  mod%02u: %08x-%08x: %s\n",
	     i, mod_start, mod_end, (char *)(_module_info_start[i].name));
      
      Space *space = new Space(Space::root());
      
      _binary.init(space->id(), mod_start, mod_end);
      int ret = read_elf(&_binary);
      if (ret)
	{
	  printf("[ldr] loading %s failed with %d\n", (char *)(_module_info_start[i].name), ret);
	  continue;
	}

      new Thread(Thread::State_ret_to_user, space, _binary.entry());
    }
}

