#pragma once

#include "elf_read.h"

/**
 * Manages all virtual memory regions of applications in this system.
 */
class Region_manager
{
  /**
   * Describes one virtual memory regions this is backed by physical
   * memory.
   */
  struct Region
    {
      unsigned id;	///< id of the address space this region belongs to
      Address phys;	///< physical address
      Address virt;	///< virtual address
      unsigned size;	///< size of the region
    };

  enum
    {
      Max_regions = 0x20,	///< total number of regions
    };

public:
  /**
   * Constructor.
   */
  Region_manager();
  /**
   * Return instance of the region manager.
   */
  static Region_manager *rm()
    {
      static Region_manager _rm;
      return &_rm;
    }
  /**
   * Add a new region.
   *
   */
  void add(unsigned id, Address phys, Address virt, unsigned size);
  /**
   * Find a physical address for a given virtual address.
   * \param id		id of the address space
   * \param virt	virtual address
   * \returns		physical address that this virtual address maps to, else zero
   */
  Address translate(unsigned id, Address virt);

private:
  unsigned _num_regions;
  Region _region_map[Max_regions];
};

//----------------------------------------------------------------------------

/**
 * Represents a user binary image to located into memory
 */
class User_binary : public Binary
{
public:
  void init(unsigned id, unsigned start, unsigned end);
  void *start();
  Mword entry()
    { return _entry; }
  void entry(unsigned entry)
    { _entry = entry; }
  void load(unsigned p_offset, unsigned p_filesz, unsigned p_paddr,
            unsigned p_vaddr, unsigned p_memsz, int type);
  Address translate(Address phys);

private:
  unsigned _id;
  unsigned _start;
  unsigned _end;
  unsigned _entry;
};

//----------------------------------------------------------------------------

/**
 * Driver class that creates user applications from binaries
 */
class Loader
{
public:
  static void exec();
};
