
package L4::ModList;
use Exporter;
use vars qw(@ISA @EXPORT);
@ISA    = qw(Exporter);
@EXPORT = qw(get_module_entry search_file get_entries);


# extract an entry with modules from a modules.list file
sub get_module_entry($$)
{
  my ($mod_file, $entry_to_pick) = @_;
  my @mods;

  if ($entry_to_pick eq 'auto-build-entry') {
    print "Automatic build entry is being built.\n";
    print "This image is useless but it always builds.\n";

    $mods[0] = { command => 'Makefile', cmdline => 'Makefile', type => 'bin'};
    $mods[1] = { command => 'Makefile', cmdline => 'Makefile', type => 'bin'};
    $mods[2] = { command => 'Makefile', cmdline => 'Makefile', type => 'bin'};

    return (
      mods    => [ @mods ],
      modaddr => 0x200000,
    );
  }

  open(M, $mod_file) || die "Cannot open $mod_file!: $!";

  # preseed first 3 modules

  my $arglen = 200;
  my $line = 0;
  my $do_entry = 0;
  my $found_entry = 0;
  my $global = 1;
  my $modaddr_title;
  my $modaddr_global;
  while (<M>) {
    $line++;
    chomp;
    s/#.*$//;
    s/^\s*//;
    next if /^$/;

    if (/^modaddr\s+(\S+)/) {
      $modaddr_global = $1 if  $global;
      $modaddr_title  = $1 if !$global;
      next;
    }

    my ($type, $remaining) = split /\s+/, $_, 2;
    $type = lc($type);

    $type = 'bin'   if $type eq 'module';

    if ($type =~ /^(entry|title)$/) {
      $do_entry = lc($entry_to_pick) eq lc($remaining);
      $found_entry |= $do_entry;
      $global = 0;
      next;
    }

    next unless $do_entry;

    my @valid_types = ( 'bin', 'data', 'bin-nostrip', 'data-nostrip',
	                'kernel', 'sigma0',
                        'module-perl', 'module-shell',
                        'bin-perl', 'bin-shell' );
    die "$line: Invalid type \"$type\""
      unless grep(/^$type$/, @valid_types);

    @m = ( $remaining );

    $type = 'perl'  if $type eq 'module-perl' or $type eq 'bin-perl';
    $type = 'shell' if $type eq 'module-shell' or $type eq 'bin-shell';

    if ($type eq 'perl' or $type eq 'shell') {

      if ($type eq 'perl') {
	@m = eval $remaining;
	die $@ if $@;
      }
      if ($type eq 'shell') {
	@m = split /\n/, `$remaining`;
	die "Shell command on line $line failed\n" if $?;
      }
      
      $type = 'bin';
    }

    foreach my $m (@m) {

      my ($file, $args) = split /\s+/, $m, 2;

      my $full = $file;
      $full .= " $args" if defined $args;
      $full =~ s/"/\\"/g;

      if (length($full) > $arglen) {
	print "$line: \"$full\" too long...\n";
	exit 1;
      }

      # special cases
      if ($type =~ /(rmgr|roottask)/i) {
	$mods[2]{command}  = $file;
	$mods[2]{cmdline}  = $full;
      } elsif ($type eq 'kernel') {
	$mods[0]{command}  = $file;
	$mods[0]{cmdline}  = $full;
      } elsif ($type eq 'sigma0') {
	$mods[1]{command}  = $file;
	$mods[1]{cmdline}  = $full;
      } else {
	push @mods, {
		      type    => $type,
		      command => $file,
		      cmdline => $full,
		    };
      }
    }
  }

  close M;

  die "Unknown entry \"$entry_to_pick\"!" unless $found_entry;

  (
    mods    => [ @mods ],
    modaddr => $modaddr_title || $modaddr_global,
  );
}

sub get_entries($)
{
  my ($mod_file) = @_;
  my @entry_list;

  open(M, $mod_file) || die "Cannot open $mod_file!: $!";

  while (<M>) {
    chomp;
    s/#.*$//;
    s/^\s*//;
    next if /^$/;
    push @entry_list, $2 if /^(entry|title)\s+(\S+)$/;
  }

  close M;

  return @entry_list;
}

# Search for a file by using a path list (single string, split with colons
# or spaces, see the split)
# return undef if it could not be found, the complete path otherwise
sub search_file($$)
{
  my $file = shift;
  my $paths = shift;

  foreach my $p (split(/:+|\s+/, $paths)) {
    return "$p/$file" if -e "$p/$file" and ! -d "$p/$file";
  }

  return $file if $file =~ /^\// && -e $file;

  undef;
}
