#
# Generic rules to build file types from other file types and other
# common functionaly that is needed to build library or program targets.
#

#
# Collect object files and avoid duplicates (by using 'sort')
#
SRC_O   = $(addprefix binary_,$(addsuffix .o,$(SRC_BIN)))
SRC     = $(sort $(SRC_C) $(SRC_CC) $(SRC_S) $(SRC_O))
OBJECTS = $(addprefix $(OBJ_PREFIX), $(addsuffix .o,$(basename $(SRC))))

#
# Make sure, that we rebuild object files after Makefile changes
#
$(wildcard $(OBJECTS)): $(filter-out $(LIB_PROGRESS_LOG),$(MAKEFILE_LIST))

INCLUDES = $(addprefix -I,$(ALL_INC_DIR))

#
# Include all dependency files we can find
#
-include *.d

$(OBJ_PREFIX)%.o: %.c
	$(MSG_COMP)$@
	$(VERBOSE)$(CC) $(CC_DEF) $(CC_OPT) $(C_FLAGS) $(INCLUDES) -c $< -o $@

$(OBJ_PREFIX)%.o: %.cc
	$(MSG_COMP)$@
	$(VERBOSE)$(CXX) $(CXX_DEF) $(CXX_OPT) $(CXX_FLAGS) $(INCLUDES) -c $< -o $@

$(OBJ_PREFIX)%.o: %.s
	$(MSG_ASSEM)$@
	$(VERBOSE)$(AS) $(AS_OPT) $(AS_FLAGS) $(INCLUDES) $< -o $@

#
# Assembler files that must be preprocessed are fed to the C compiler.
#
$(OBJ_PREFIX)%.o: %.S
	$(MSG_COMP)$@
	$(VERBOSE)$(CC) $(CC_DEF) $(CC_OPT) $(AS_FLAGS) -D__ASSEMBLY__ $(INCLUDES) -c $< -o $@

#
# Link binary data
#
# We transform binary data into intermediate C
# source code by using 'hexdump' and 'sed'. This
# code gets then transformed into an object file via
# C compiler. This mechanism is more generic than the
# use of 'ld -r -oformat default -b binary', which is
# not supported on some systems such as Mac OS X.
# Furthermore, we are able to exploit an 'gcc' attribute
# for aligning linked binary data, which is required on
# some architectures (e.g., ARM) but impossible to
# enforce by using 'ld' directly.
#

symbol_name = _binary_$(subst .,_,$(subst $(OBJ_PREFIX)binary_,,$(subst .o,,$@)))

$(OBJ_PREFIX)binary_%.o: %
	$(MSG_CONVERT)$@
	$(VERBOSE)hexdump -ve '/1 "0x%02x,"' $< |\
	  sed '1s/^/char $(symbol_name)_start[] __attribute__((aligned (4))) = {/;$$s/,$$/};/;$$a\' |\
	    $(CC) -x c -o $@ -c -
